/***************************************************************************//**
 *   @file   AD7980 sketch.
 *   @brief  Demo project for AD7980 using MPIDE Serial Monitor.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 699
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include <AD7980.h>
#include <DSPI.h>

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/*! List of available commands */
const char* commandsList[] = {"help?",
                              "voltage?",
                              "register?"};
const char* commandsDescription[] = {
    "  -  Diplays all available commands",
    "  -  Initiates a conversion and displays the captured voltage.",
    "  -  Initiates a conversion and displays the raw data in decimal format."};

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
AD7980 myAD7980;
/*! Variables used for console operations */
char   commandsNumber = (sizeof(commandsList) / sizeof(const char*));
char   receivedCommand[20];
char   invalidCommand = 0;
char   commandType = 0;
char   command = 0;
char   displayCommand = 0;
double commandParam = 0;
/*! Temporary variables */
char   tempString [10]  = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
short  registerValue = 0;
double result = 0;

/***************************************************************************//**
 * @brief Reads one command from UART.
 *
 * @param command - Read command.
 *
 * @return None.
*******************************************************************************/
void CONSOLE_GetCommand(char* command)

{
    unsigned char receivedChar = 0;
    
    unsigned char charNumber = 0;
    
    while(receivedChar != 0x0D)
    
    {
      if (Serial.available() > 0)
      {
        
        receivedChar = Serial.read();
        command[charNumber++] = receivedChar; 
      }
    }
    command[charNumber] = 0;
}

/***************************************************************************//** 
 * @param receivedCommand - Received command.
 * @param expectedCommand - Expected command.
 * @param commandParameter - Command parameter.
 *
 * @return commandType - Type of the command.
 *                       Example: 0 - Commands don't match.
 *                                1 - Write command.
 *                                2 - Read command.
*******************************************************************************/
unsigned char CONSOLE_CheckCommands(char* receivedCommand,
                                    const char* expectedCommand,
                                    double* commandParameter)
{
    unsigned char commandType = 1;
    unsigned char charIndex = 0;

    unsigned char parameterString[5] = {0, 0, 0, 0, 0};
    unsigned char parameterIndex = 0;

    while((expectedCommand[charIndex] != '?') &&
          (expectedCommand[charIndex] != '=') &&
          
          (commandType != 0))
    {
        if(expectedCommand[charIndex] != receivedCommand[charIndex])
        {
            commandType = 0;
        }
        charIndex++;
    }
    if(commandType != 0)
    {
        if(expectedCommand[charIndex] == '=')
        {
            if(receivedCommand[charIndex] == '=')
            {
                charIndex++;
     
                while(receivedCommand[charIndex] != 0x0D)
                {
                    parameterString[parameterIndex] = receivedCommand[charIndex];
                    charIndex++;
                    parameterIndex++;
                    
                }
                *commandParameter = atof((const char*)parameterString);
            }
            else
            {
                commandType = 0;
            }
        }
        if(expectedCommand[charIndex] == '?')
        {
            if(receivedCommand[charIndex] == '?')
            {
                    commandType = 2;
            }
            else
            {
                    commandType = 0;
            }
        }
    }

    return commandType;
}

/***************************************************************************//**
 * @brief Setup function.
 *
 * @return none.
*******************************************************************************/ 
void setup()
{
  Serial.begin(9600);
  if(myAD7980.Init())
  {
      Serial.println("AD7980 OK");
  }
  else
  {
      Serial.println("AD7980 Error");
  } 
}

/***************************************************************************//**
 * @brief Loop function.
 *
 * @return none.
*******************************************************************************/
void loop()
{
  /*! Wait a command from the user. */
  CONSOLE_GetCommand(receivedCommand);
  invalidCommand = 0;
  /*! Check if the received command identifies as one of the command list.  */
  for(command = 0; command < commandsNumber; command++)
  {
      commandType = CONSOLE_CheckCommands(receivedCommand,
                                          commandsList[command],
                                          (double*)&commandParam);
      /*! Count the commands that are invalid. */
      if(commandType == 0)
      {
          invalidCommand++;
      }
      else
      {
        switch(command)
        {
          case 0:                                        /*!< "help?" command*/
              Serial.println("Available commands:");
              for(displayCommand = 0; displayCommand < commandsNumber;
                  displayCommand++)
              {
                  Serial.print(commandsList[displayCommand]);
                  Serial.println(commandsDescription[displayCommand]);
              }
              break;
          case 1:                                        /*!< "voltage?" command*/
              result = myAD7980.ReadVoltage();
              /*! Send the requested value to user */
              Serial.print("voltage=");
              /*! Convert float to string and send it to user */
              sprintf(tempString, "%.3f", result);
              Serial.print(tempString);
              Serial.println(" [V]");
              break;
          case 2:                                        /*!< "register?" command*/
              registerValue = myAD7980.ReadRawData();
              /*! Send the requested value to user */
              Serial.print("register=");
              itoa(registerValue, tempString, 10);
              Serial.println(tempString);
              break;
         default:
              break;   
        }/*!< End switch */   
    }/*!< End else */
  }/*!< End for */
  if(invalidCommand == commandsNumber)
  {
      /*! Send feedback to user */
       Serial.println("Invalid command");
  }
}

